@@ -16,19 +16,36 @@
 #    more: https://github.com/manatlan/guy
 # #############################################################################
 
-import guy,sys,asyncio,hashlib,html
+import guy,sys,asyncio,hashlib,html,inspect,types
 import typing as T
 
 
-_gg=lambda x: x.get() if isinstance(x,ReactiveProp) else x #TODO: rename to value() ?
+isAsyncGenerator=lambda x: "async_generator" in str(type(x)) #TODO: howto better ?
+
+value=lambda x: x.get() if isinstance(x,ReactiveProp) else x
+
+def convjs(l:list)->list:
+    ll=[]
+    for i in l:
+        if i is None:
+            ll.append("null")
+        elif isinstance(i,bool):
+            ll.append( i and "true" or "false")
+        elif isinstance(i,bytes):   # bind b"" -> js
+            ll.append( i.decode() )
+        elif isinstance(i,str):
+            ll.append( "'%s'" % html.escape(i))
+        else:
+            ll.append( str(i) )
+    return ll
 
 def log(*a):
-    # print(*a)
+    #~ print(*a)
     pass
 
 
 class MyMetaclass(type):
-    def __getattr__(self,name:str) -> any:
+    def __getattr__(self,name:str) -> callable:
         def _(*a,**k) -> Tag:
             t=Tag(*a,**k)
             t.tag=name
@@ -36,7 +53,6 @@ class MyMetaclass(type):
         return _
 
 class Tag(metaclass=MyMetaclass):
-    __metaclass__ = MyMetaclass
     """ This is a helper to produce a "HTML TAG" """
     tag="div" # default one
     klass=None
@@ -73,6 +89,7 @@ class Tag(metaclass=MyMetaclass):
         return "<%s>" % self.__class__.__name__
 
 
+
 class CSS(Tag):
     def __init__(self,content):
         self._md5=hashlib.md5("css:".encode()+content.encode()).hexdigest()
@@ -103,26 +120,26 @@ class ReactiveProp:
 
 
     def __eq__(self, v):
-        return self.get() == _gg(v)
+        return self.get() == value(v)
 
     def __ne__(self, v):
-        return self.get() != _gg(v)
+        return self.get() != value(v)
 
     def __lt__(self, v):
-        return self.get() < _gg(v)
+        return self.get() < value(v)
 
     def __le__(self, v):
-        return self.get() <= _gg(v)
+        return self.get() <= value(v)
 
     def __ge__(self, v):
-        return self.get() >= _gg(v)
+        return self.get() >= value(v)
 
     def __gt__(self, v):
-        return self.get() > _gg(v)
+        return self.get() > value(v)
 
 
     def __iadd__(self,v):
-        vv=self.get() + _gg(v)
+        vv=self.get() + value(v)
         self.set( vv )
         return self
 
@@ -139,31 +156,28 @@ class ReactiveProp:
     #TODO: add a lot of __slot__ ;-)
 
 
-class ReactiveMethod:
-    """ like ReactiveProp, but for gtag.method wchich can return binded tag
-        (object created by @bind decorator)
+
+def local( method ): # gtag.event decorator
+    """ Decorator to make a gtag.method() able to start after init !
     """
-    def __init__(self,instance,method,args,kargs):
-        self.__instance=instance
+    Capacity(method).set(Capacity.LOCAL)
+    return method
+
+class Capacity:
+    LOCAL="local"
+    def __init__(self,method:callable):
         self.__method=method
-        self.__a=args
-        self.__k=kargs
+    def has( self, capacity ):
+        if hasattr(self.__method,"capacities"):
+            return capacity in self.__method.capacities
+    def set( self, capacity ):
+        if not hasattr(self.__method,"capacities"):
+            self.__method.capacities=[]
+        self.__method.capacities.append(capacity)
+    @property
+    def hasLocal( self ):
+        return self.has(Capacity.LOCAL)
 
-    def __call__(self):
-        return self.__method(self.__instance,*self.__a,**self.__k)
-
-    def __str__(self) -> str:
-        return str(self())
-
-
-def bind( method ): # gtag.method decorator -> ReactiveMethod
-    """ Decorator to make a gtag.method() able to return a "Reactive Tag" !
-        (like 'computed vars' in vuejs)
-    """
-    def _(gtagInstance,*a,**k):
-        assert isinstance(gtagInstance,GTag)
-        return ReactiveMethod(gtagInstance,method,a,k)
-    return _
 
 
 class GtagProxy:
@@ -195,15 +209,18 @@ class GTag:
     The magic thing ;-)
     """
     size=None
+    _call=None # first event to call at start !
     """ size of the windowed runned gtag (tuple (width,height) or guy.FULLSCREEN or None) """
 
     # implicit parent version (don't need to pass self(=parent) when creating a gtag)
     def __init__(self,*a,**k):
         self._tag=None
+        self.script=None
 
-        if "dontGuessParent" in k.keys(): # clonage (only main tags, so parent is None)
-            del k["dontGuessParent"]
-            parent=None
+        if "parent" in k.keys(): # clonage (only main tags, so parent is None)
+            #(but could be used to reparent a gtag manually)
+            parent=k["parent"]
+            del k["parent"]
         else:
             # guess parent
             frame = sys._getframe(1)
@@ -225,23 +242,21 @@ class GTag:
         log("INIT",repr(self))
         self.init(*self._args,**self._kargs)
 
-        self._tag = self.build() # can be a Tag or ReactiveMethod
+        self._tag = self.build()
 
         # Store the instance in the parent._childs
         if self._parent:
             self._parent._childs.append(self)
 
-
-
     def _tree(self):
-        assert self._parent is None,"You are not on the main instance, you can't get tree"
         def _gc(g,lvl=0) -> list:
             ll=["+" + ("   "*lvl) + repr(g)]
             for obj in g.innerChilds:
                 inners=_gc(obj,lvl+1)
                 ll.extend( [i+' (INNER)' for i in inners] )
             for obj in g._childs:
-                ll.extend( _gc(obj,lvl+1) )
+                if obj not in g.innerChilds:
+                    ll.extend( _gc(obj,lvl+1) )
             return ll
         return "\n".join(_gc(self,0))
 
@@ -251,14 +266,14 @@ class GTag:
         return [v for k,v in self.__dict__.items() if k not in ["_tag","_parent"] and isinstance(v,GTag)]
 
     def _getChilds(self) -> dict:
-        assert self._parent is None,"You are not on the main instance, you can't get a child"
 
         def _gc(g) -> dict:
             d={g.id:g}
             for obj in g.innerChilds:
                 d.update( _gc(obj) )
             for obj in g._childs:
-                d.update( _gc(obj) )
+                if obj not in g.innerChilds:
+                    d.update( _gc(obj) )
             return d
 
         return _gc(self)
@@ -293,6 +308,7 @@ class GTag:
         return GtagProxy( self._getMain() )
 
 
+
     @property
     def bind(self) -> any:
         """ to bind attribute or method !"""
@@ -307,7 +323,7 @@ class GTag:
                 elif name in dir(self):   # bind a self.method    -> return a js/string for a guy's call in js side
                     def _(*args):
                         if args:
-                            return "self.bindUpdate('%s',GID,'%s',%s)" % (self.id,name,",".join([str(i) for i in args]) ) #TODO: escaping here ! (and the render/str ?) json here !
+                            return "self.bindUpdate('%s',GID,'%s',%s)" % (self.id,name,",".join(convjs(list(args))) ) #TODO: escaping here ! (and the render/str ?) json here !
                         else:
                             return "self.bindUpdate('%s',GID,'%s')" % (self.id,name)
                     return _
@@ -316,11 +332,13 @@ class GTag:
         return Binder()
 
 
-    def _clone(self):
-        props={k:v for k,v in self.__dict__.items() if k[0]!="_"}
-        gtag = self.__class__(*self._args,**self._kargs,dontGuessParent=True)
+    def _clone(self): #TODO: not clear here ... need redone
+        assert self._parent==None,"Can't clone a gtag which is not the main"
+        props={k:v for k,v in self.__dict__.items() if k[0]!="_" or k=="_call"}
+        gtag = self.__class__(*self._args,**self._kargs,parent=None)
         gtag.__dict__.update(props)
-        assert isinstance(gtag,GTag)
+        gtag.init(*self._args,**self._kargs)
+        gtag._rebuild()
         log("^^^ CLONED ^^^",repr(self),"-->",repr(gtag))
         return gtag
 
@@ -359,30 +377,15 @@ class GTag:
         """
         pass
 
-    def script(self):
-        """ Override to get back some js code"""
-        pass
+    def exit(self,v=None): pass # overriden by run/runcef/serve
 
+    def _rebuild(self):
+        self._childs=[]
+        self._tag=self.build()
 
     def __str__(self):
         log("___rendering",repr(self))
         o= self._tag
-        if isinstance(o,ReactiveMethod):
-            # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-            # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-            # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-            # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-            # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-            # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-            self._childs=[] # clear the (builded) childs (from build()) .
-            # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-            # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-            # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-            # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-            # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-            # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-            o=o()
-
         if o is None:
             return ""
         else:
@@ -392,21 +395,11 @@ class GTag:
                 assert isinstance(o._tag,Tag)
                 o=o._tag        #TODO: recursivity here ?!... wtf if gtag return a gtag
                 o.id=self.id
-            else:
-                raise Exception("%s --build--> ???%s???" % (repr(self),type(o)))
             return str(o)
 
     def __repr__(self):
-        if self._tag:
-            if isinstance(self._tag,ReactiveMethod):
-                s="DYNAMIC"
-            else:
-                s="STATIC"
-        else:
-            s="???"
-        return "<GTag: %s [%s] [parent:%s] (innerchilds=%s)>" % (
+        return "<GTag: %s [parent:%s] (innerchilds=%s)>" % (
             self.id,
-            s,
             self._parent.id if self._parent else "None",
             [i.id for i in self.innerChilds]
         )
@@ -414,39 +407,41 @@ class GTag:
     def __setattr__(self,k,v):
         o=self.__dict__.get(k)
         if isinstance(o,ReactiveProp):
-            o.set( _gg(v) )
+            o.set( value(v) )
         else:
             super().__setattr__(k,v)
 
     def _getScripts(self) -> str:
         ll=[]
-        for v in self._getChilds().values():
-            js=v and v.script() or None
+        for g in self._getChilds().values():
+            js=g.script
             if js:
-                if isinstance(js,ReactiveMethod): js=js() # dangerous ?
-                ll.append( "(function(tag){%s})(document.getElementById('%s'))" % (str(js),v.id) )
+                ll.append( "(function(tag){%s})(document.getElementById('%s'))" % (str(js),g.id) )
         return ";".join(ll)
 
 
-    def update(self) -> dict:
+    def _update(self) -> dict:
         h=str(self)
         s=self._getScripts()
         log(">>>UPDATE:",repr(self))
         log(self._tree())
-        return dict(script="""document.querySelector("#%s").innerHTML=`%s`;%s""" % (
+        return dict(script="""document.querySelector("#%s").outerHTML=`%s`;%s""" % (
             self.id, h,s
         ))
 
-    def run(self,*a,**k) -> any:
+    def run(self,*a,start=None,**k) -> any:
         """ Run as Guy App (using Chrome) """
+        self._call=start
         return GTagApp(self,False).run(*a,**k)
 
-    def runCef(self,*a,**k) -> any:
+    def runCef(self,*a,start=None,**k) -> any:
         """ Run as Guy App (using Cef) """
+        self._call=start
         return GTagApp(self,False).runCef(*a,**k)
 
-    def serve(self,*a,**k) -> any:
+    def serve(self,*a,start=None,**k) -> any:
         """ Run as Guy Server App """
+        self._call=start
         return GTagApp(self,True).serve(*a,**k)
 
 
@@ -478,58 +473,92 @@ class GTagApp(guy.Guy):
             if(!sessionStorage["gtag"]) sessionStorage["gtag"]=Math.random().toString(36).substring(2);
             var GID=sessionStorage["gtag"];
 
+            async function forceUpdateManually(x) {console.log(x),eval(x)}
+
             async function getSessionId() {return GID}
             async function _render(html,script) {
                 document.body.innerHTML=html;
                 if(script) eval(script)
+                console.log(script)
             }
-            /*function update() { return self.update(GID) } promise */
         </script>
         %s
     </head>
     <body>
             <script src="guy.js"></script>
     </body>
-</html>
-        """ % "\n".join([str(h) for h in hh])
+</html>""" % "\n".join([str(h) for h in hh])
 
     async def init(self):
-        if self._ses is not None:
+        if self._ses is not None: # web mode
             gid=await self.js.getSessionId()
             log("CREATE SESSION:",gid)
             gtag = self._ses.get(gid)
             if gtag is None:
                 gtag = self._originalGTag._clone()
                 self._ses[gid] = gtag
-        else:
+        else: # app mode
             gtag = self._originalGTag
 
         gtag.exit = self.exit
 
         log(">>>SERVE",repr(gtag))
         log(gtag._tree())
-        await self.js._render( str(gtag), gtag._getScripts() )
+
+        #=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- call statement (old autostart system)
+        caller=""
+        if gtag._call:  # there is an event to call at start !
+            if isAsyncGenerator(gtag._call) or asyncio.iscoroutine(gtag._call):
+
+                if asyncio.iscoroutine(gtag._call):
+                    fname = gtag._call.__name__
+                    args = gtag._call.cr_frame.f_locals  # dict object
+                elif isAsyncGenerator(gtag._call):
+                    fname=gtag._call.__name__
+                    args = gtag._call.ag_frame.f_locals  # dict object
+
+                if "self" in args: del args["self"]
+                args=args.values()
+
+                method=getattr(gtag.bind,fname)
+                caller=method(*args)
+                print("====>",gtag._call,caller)
+        #=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
+
+        await self.js._render( str(gtag), gtag._getScripts()+";"+caller )
+
+    async def forceUpdate(self,g): #can't be called from client side !
+        g._rebuild()
+        js=g._update()["script"]    #TODO: not cute ... should unify way to render to client (currently: 3 ways)
+        log(">>>Force UPDATE:",repr(g))
+        await self.js.forceUpdateManually(js)
 
     async def bindUpdate(self,id:str,gid:str,method:str,*args):
         """ inner (js exposed) guy method, called by gtag.bind.<method>(*args) """
         if self._ses is None:
             gtag=self._originalGTag
-            gid=None
         else:
             gtag=self._ses[gid]
 
         #////////////////////////////////////////////////////////////////// THE MAGIC TODO: move to gtag
         obj=gtag._getRef(id)
-        # keep the main tag, and the current object !
-        # (others will be rebuild during rendering)
 
         log("BINDUPDATE on",repr(gtag),"----->",repr(obj),"%s(%s)"% (method,args))
         proc=getattr(obj,method)
+        toRender=gtag if not Capacity(proc).hasLocal else obj
 
         if asyncio.iscoroutinefunction( proc ):
-            r=await proc(*args)
+            rep=await proc(*args)
         else:
-            r=proc(*args)
+            rep=proc(*args)
 
+        if rep:
+            if isAsyncGenerator(rep):
+                async for _ in rep: # could use yielded thing to update all or local ?!
+                    await self.forceUpdate(toRender)
+            else:
+                raise Exception("wtf?")
+
+        toRender._rebuild()
+        return toRender._update() #UPDATE ALL (historic way)
         #////////////////////////////////////////////////////////////////// THE MAGIC
-        return gtag.update() #could update the obj gtag only !
